@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3001 $
+//  $Revision: 3647 $
 //
-//  $LastChangedDate: 2011-03-16 12:14:22 -0700 (Wed, 16 Mar 2011) $
+//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -201,7 +201,7 @@ ParseCommandLine ( const int argc, const char* argv[] )
     
     if ( SymmetryParametersFile ) 
       {
-      cmtk::ClassStream inStream( SymmetryParametersFile, cmtk::ClassStream::READ );
+      cmtk::ClassStream inStream( SymmetryParametersFile, cmtk::ClassStream::MODE_READ );
       if ( inStream.IsValid() ) 
 	{
 	cmtk::ParametricPlane *plane = NULL;
@@ -479,7 +479,7 @@ doMain ( const int argc, const char* argv[] )
   
   if ( SymmetryOutFileName )
     {
-    cmtk::ClassStream stream( SymmetryOutFileName, cmtk::ClassStream::WRITE );
+    cmtk::ClassStream stream( SymmetryOutFileName, cmtk::ClassStream::MODE_WRITE );
     stream << parametricPlane;
     stream.Close();
     }
