@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3197 $
+//  $Revision: 3878 $
 //
-//  $LastChangedDate: 2011-05-09 16:50:08 -0700 (Mon, 09 May 2011) $
+//  $LastChangedDate: 2012-02-15 15:55:31 -0800 (Wed, 15 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -316,6 +316,14 @@ public:
   /**\name Direct parameter modifications.
    */
   //@{
+  /// Set transformation's translation vector.
+  void SetTranslation( const Self::SpaceVectorType& delta ) 
+  { 
+    for ( int dim = 0; dim < 3; ++dim ) 
+      this->m_Parameters[dim] = delta[dim];
+    this->ComposeMatrix();
+  }
+  
   /// Set transformation's translation vector.
   void SetXlate ( const Types::Coordinate* xlate ) 
   { 
@@ -355,8 +363,9 @@ public:
   }
 
   /// Get scale factors with implicit conversion of log scales.
-  template<class T> void GetScales( T (&scales)[3] ) const
+  FixedVector<3,Types::Coordinate> GetScales() const
   { 
+    FixedVector<3,Types::Coordinate> scales;
     if ( this->m_LogScaleFactors )
       {
       for ( size_t i = 0; i < 3; ++i )
@@ -371,6 +380,7 @@ public:
 	scales[i] = this->m_Parameters[6+i];
 	}
       }
+    return scales;
   }
 
   /// Set transformation's scaling factors.
@@ -396,6 +406,14 @@ public:
     this->ComposeMatrix();
   }
 
+  /// Set transformation's rotation, scaling, and shearing center.
+  void SetCenter( const Self::SpaceVectorType& center ) 
+  {
+    for ( size_t dim = 0; dim < 3; ++dim )
+      this->m_Parameters[12+dim] = center[dim];
+    this->ComposeMatrix();
+  }
+
   /// Set transformation's rotation, scaling, and shearing center.
   void SetCenter ( const Types::Coordinate* center ) 
   {
