@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3347 $
+//  $Revision: 3665 $
 //
-//  $LastChangedDate: 2011-08-09 15:45:16 -0700 (Tue, 09 Aug 2011) $
+//  $LastChangedDate: 2011-12-23 10:36:47 -0800 (Fri, 23 Dec 2011) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -43,7 +43,7 @@ cmtk
 {
 
 DataGridFilter
-::DataGridFilter( DataGrid::SmartPtr dataGrid )
+::DataGridFilter( DataGrid::SmartConstPtr dataGrid )
   : m_DataGrid( dataGrid )
 {
   if ( !this->m_DataGrid->GetData() )
@@ -392,31 +392,49 @@ DataGridFilter::GetDataKernelFiltered
   const std::vector<Types::DataItem>& filterY,
   const std::vector<Types::DataItem>& filterZ ) const
 {
-  TypedArray::SmartPtr result = TypedArray::Create( this->m_DataGrid->GetData()->GetType(), this->m_DataGrid->GetNumberOfPixels() );
+  // create and initialize result (data are copied at this point from the source array)
+  TypedArray::SmartPtr result( this->m_DataGrid->GetData()->Clone() );
   
   ThreadPool& threadPool = ThreadPool::GetGlobalThreadPool();
   const size_t numberOfTasks = 4 * threadPool.GetNumberOfThreads() - 3;
 
+  // initialize common thread data
   std::vector<FilterThreadParameters> params( numberOfTasks );
   for ( size_t task = 0; task < numberOfTasks; ++task )
     {
     params[task].thisObject = this;
-    params[task].m_Filter = &filterX;
     params[task].m_Result = result;
     }
-  threadPool.Run( GetFilteredDataThreadX, params );
 
-  for ( size_t task = 0; task < numberOfTasks; ++task )
+  // run x filter
+  if ( filterX.size() > 1 )
     {
-    params[task].m_Filter = &filterY;
+    for ( size_t task = 0; task < numberOfTasks; ++task )
+      {
+      params[task].m_Filter = &filterX;
+      }
+    threadPool.Run( GetFilteredDataThreadX, params );
     }
-  threadPool.Run( GetFilteredDataThreadY, params );
 
-  for ( size_t task = 0; task < numberOfTasks; ++task )
+  // run y filter
+  if ( filterY.size() > 1 )
     {
-    params[task].m_Filter = &filterZ;
+    for ( size_t task = 0; task < numberOfTasks; ++task )
+      {
+      params[task].m_Filter = &filterY;
+      }
+    threadPool.Run( GetFilteredDataThreadY, params );
+    }
+  
+  // run z filter
+  if ( filterZ.size() > 1 )
+    {
+    for ( size_t task = 0; task < numberOfTasks; ++task )
+      {
+      params[task].m_Filter = &filterZ;
+      }
+    threadPool.Run( GetFilteredDataThreadZ, params );
     }
-  threadPool.Run( GetFilteredDataThreadZ, params );
   
   return result;
 }
@@ -445,7 +463,7 @@ DataGridFilter
       // copy row data to buffer
       size_t ofs = ThisConst->m_DataGrid->GetOffsetFromIndex( 0, y, z );
       for ( int x=0; x < dims[0]; ++x, ++ofs )
-	if ( !ThisConst->m_DataGrid->GetDataAt( pixelBufferFrom[x], ofs ) )
+	if ( !result->Get( pixelBufferFrom[x], ofs ) )
 	  pixelBufferFrom[x] = 0;
       
       // convolve row with filter
