@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3690 $
+//  $Revision: 3798 $
 //
-//  $LastChangedDate: 2012-01-04 14:59:46 -0800 (Wed, 04 Jan 2012) $
+//  $LastChangedDate: 2012-02-02 14:44:12 -0800 (Thu, 02 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -86,7 +86,7 @@ public:
   };
   
   /// Initialization constructor.
-  explicit FixedVector( const typename Self::Init& init )
+  FixedVector( const typename Self::Init& init )
   {
     std::fill( this->begin(), this->end(), init.m_Value );
   }
@@ -375,6 +375,17 @@ Min( const FixedVector<NDIM,T>& lhs, const FixedVector<NDIM,T>& rhs )
   return result;
 }
 
+/// Less-than operator.
+template<size_t NDIM,typename T>
+bool
+operator<( const FixedVector<NDIM,T>& lhs, const FixedVector<NDIM,T>& rhs )
+{
+  for ( size_t i = 0; i < NDIM; ++i )
+    if ( ! (lhs[i] < rhs[i] ) )
+      return false;
+  return true;
+}
+
 /// Stream input operator.
 template<size_t NDIM,typename T>
 std::ostream& operator<<( std::ostream& stream, const FixedVector<NDIM,T>& index )
