@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3517 $
+//  $Revision: 3871 $
 //
-//  $LastChangedDate: 2011-10-27 12:42:15 -0700 (Thu, 27 Oct 2011) $
+//  $LastChangedDate: 2012-02-15 14:39:46 -0800 (Wed, 15 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -52,65 +52,35 @@ SplineWarpXform::SplineWarpXform()
 
 void SplineWarpXform::Init () 
 {
-  this->GlobalScaling = 1.0;
+  this->m_GlobalScaling = 1.0;
 }
 
 SplineWarpXform::SplineWarpXform 
-( const FixedVector<3,Types::Coordinate>& domain, const Types::Coordinate delta, const AffineXform* initialXform, const bool exactDelta  )
+( const Self::SpaceVectorType& domain, const Types::Coordinate delta, const AffineXform* initialXform, const bool exactDelta  )
 {
   this->Init( domain, delta, initialXform, exactDelta );
 }
 
-void
-SplineWarpXform
-::Init
-( const Self::SpaceVectorType& domain, const Types::Coordinate delta, const AffineXform* initialXform, const bool exactDelta  )
-{
-  this->Init();
-  this->Domain = domain;
-  this->m_InitialAffineXform = initialXform->Clone();
-
-  if ( exactDelta ) 
-    {
-    for ( int dim=0; dim<3; ++dim ) 
-      {
-      Spacing[dim] = delta;
-      this->m_Dims[dim] = static_cast<int>( 4 + (Domain[dim] / Spacing[dim]) );
-      Domain[dim] = (this->m_Dims[dim] - 3) * Spacing[dim];
-      }
-    } 
-  else
-    {
-    for ( int dim=0; dim<3; ++dim )
-      this->m_Dims[dim] = 2 + std::max( 2, 1+static_cast<int>( domain[dim]/delta ) );
-    }
-  
-  NumberOfControlPoints = this->m_Dims[0] * this->m_Dims[1] * this->m_Dims[2];
-  this->AllocateParameterVector( 3 * NumberOfControlPoints );
-  
-  this->Update( exactDelta );
-  this->InitControlPoints( this->m_InitialAffineXform );
-}
-
 SplineWarpXform::SplineWarpXform
-( const FixedVector<3,Types::Coordinate>& domain, const Self::IndexType& dims, CoordinateVector::SmartPtr& parameters, const AffineXform* initialXform )
+( const Self::SpaceVectorType& domain, const Self::ControlPointIndexType& dims, CoordinateVector::SmartPtr& parameters, const AffineXform* initialXform )
 {
   this->Init();
-  this->Domain = domain;
+  this->m_Domain = domain;
   this->m_Dims = dims;
 
   if ( initialXform )
     {
     this->m_InitialAffineXform = initialXform->Clone();
-    GlobalScaling = this->m_InitialAffineXform->GetGlobalScaling();
+    this->m_GlobalScaling = this->m_InitialAffineXform->GetGlobalScaling();
     } 
   else
     {
     this->m_InitialAffineXform = AffineXform::SmartPtr( NULL );
+    this->m_GlobalScaling = 1.0;
     }
 
-  NumberOfControlPoints = this->m_Dims[0] * this->m_Dims[1] * this->m_Dims[2];
-  this->m_NumberOfParameters = 3 * NumberOfControlPoints;
+  this->m_NumberOfControlPoints = this->m_Dims[0] * this->m_Dims[1] * this->m_Dims[2];
+  this->m_NumberOfParameters = 3 * this->m_NumberOfControlPoints;
 
   if ( !parameters )
     this->m_ParameterVector = CoordinateVector::SmartPtr( new CoordinateVector( this->m_NumberOfParameters ) );
@@ -124,18 +94,58 @@ SplineWarpXform::SplineWarpXform
     this->InitControlPoints( this->m_InitialAffineXform );
 }
 
+void
+SplineWarpXform
+::Init
+( const Self::SpaceVectorType& domain, const Types::Coordinate delta, const AffineXform* initialXform, const bool exactDelta  )
+{
+  this->Init();
+  this->m_Domain = domain;
+  if ( initialXform )
+    {
+    this->m_InitialAffineXform = initialXform->Clone();
+    this->m_GlobalScaling = this->m_InitialAffineXform->GetGlobalScaling();
+    }
+  else
+    {
+    this->m_InitialAffineXform = AffineXform::SmartPtr( NULL );
+    this->m_GlobalScaling = 1.0;
+    }
+
+  if ( exactDelta ) 
+    {
+    for ( int dim=0; dim<3; ++dim ) 
+      {
+      this->m_Spacing[dim] = delta;
+      this->m_Dims[dim] = static_cast<int>( 4 + (this->m_Domain[dim] / this->m_Spacing[dim]) );
+      this->m_Domain[dim] = (this->m_Dims[dim] - 3) * this->m_Spacing[dim];
+      }
+    } 
+  else
+    {
+    for ( int dim=0; dim<3; ++dim )
+      this->m_Dims[dim] = 2 + std::max( 2, 1+static_cast<int>( domain[dim]/delta ) );
+    }
+  
+  this->m_NumberOfControlPoints = this->m_Dims[0] * this->m_Dims[1] * this->m_Dims[2];
+  this->AllocateParameterVector( 3 * this->m_NumberOfControlPoints );
+  
+  this->Update( exactDelta );
+  this->InitControlPoints( this->m_InitialAffineXform );
+}
+
 void
 SplineWarpXform::InitControlPoints( const AffineXform* affineXform )
 {
   Types::Coordinate *ofs = this->m_Parameters;
-  Types::Coordinate pZ = -Spacing[2];
-  for ( int z=0; z<this->m_Dims[2]; ++z, pZ+=Spacing[2] ) 
+  Types::Coordinate pZ = -this->m_Spacing[2];
+  for ( int z=0; z<this->m_Dims[2]; ++z, pZ+=this->m_Spacing[2] ) 
     {
-    Types::Coordinate pY = -Spacing[1];
-    for ( int y=0; y<this->m_Dims[1]; ++y, pY+=Spacing[1] ) 
+    Types::Coordinate pY = -this->m_Spacing[1];
+    for ( int y=0; y<this->m_Dims[1]; ++y, pY+=this->m_Spacing[1] ) 
       {
-      Types::Coordinate pX = -Spacing[0];
-      for ( int x=0; x<this->m_Dims[0]; ++x, pX+=Spacing[0], ofs+=3 ) 
+      Types::Coordinate pX = -this->m_Spacing[0];
+      for ( int x=0; x<this->m_Dims[0]; ++x, pX+=this->m_Spacing[0], ofs+=3 ) 
 	{
 	ofs[0] = pX;
 	ofs[1] = pY;
@@ -147,7 +157,7 @@ SplineWarpXform::InitControlPoints( const AffineXform* affineXform )
   if ( affineXform ) 
     {
     ofs = this->m_Parameters;
-    for ( unsigned int idx = 0; idx < NumberOfControlPoints; ++idx, ofs+=3 ) 
+    for ( unsigned int idx = 0; idx < this->m_NumberOfControlPoints; ++idx, ofs+=3 ) 
       {
       Self::SpaceVectorType p( ofs );
       affineXform->ApplyInPlace( p );
@@ -156,15 +166,25 @@ SplineWarpXform::InitControlPoints( const AffineXform* affineXform )
       ofs[2] = p[2];
       }
     
-    affineXform->GetScales( this->InverseAffineScaling );
-    GlobalScaling = affineXform->GetGlobalScaling();
+    this->m_InverseAffineScaling = affineXform->GetScales();
+    this->m_GlobalScaling = affineXform->GetGlobalScaling();
     } 
   else
     {
-    InverseAffineScaling[0] = InverseAffineScaling[1] = InverseAffineScaling[2] = GlobalScaling = 1.0;
+    this->m_InverseAffineScaling[0] = this->m_InverseAffineScaling[1] = this->m_InverseAffineScaling[2] = this->m_GlobalScaling = 1.0;
     }
 }
 
+SplineWarpXform::ControlPointRegionType 
+SplineWarpXform::GetInsideControlPointsRegion() const
+{
+  Self::ControlPointRegionType region = this->GetAllControlPointsRegion();
+  region.From().AddScalar( 1 );
+  region.To().AddScalar( -1 );
+  
+  return region;
+}
+
 void
 SplineWarpXform::Update 
 ( const bool exactDelta ) 
@@ -176,14 +196,14 @@ SplineWarpXform::Update
     assert( this->m_Dims[dim] > 3 );
     if ( exactDelta ) 
       {
-      InverseSpacing[dim] = 1.0 / Spacing[dim];
+      this->m_InverseSpacing[dim] = 1.0 / this->m_Spacing[dim];
       } 
     else
       {
-      Spacing[dim] = Domain[dim] / (this->m_Dims[dim]-3);
-      InverseSpacing[dim] = 1.0*(this->m_Dims[dim]-3) / Domain[dim];
+      this->m_Spacing[dim] = this->m_Domain[dim] / (this->m_Dims[dim]-3);
+      this->m_InverseSpacing[dim] = 1.0*(this->m_Dims[dim]-3) / this->m_Domain[dim];
       }
-    m_Offset[dim] = -Spacing[dim];
+    m_Offset[dim] = -this->m_Spacing[dim];
     }
   
   int dml = 0;
@@ -201,12 +221,12 @@ SplineWarpXform::CloneVirtual() const
   newXform->m_ParameterVector = CoordinateVector::SmartPtr( this->m_ParameterVector->Clone() );
   newXform->m_Parameters = newXform->m_ParameterVector->Elements;
   newXform->m_NumberOfParameters = this->m_NumberOfParameters;
-  newXform->NumberOfControlPoints = this->NumberOfControlPoints;
+  newXform->m_NumberOfControlPoints = this->m_NumberOfControlPoints;
   
   newXform->m_Dims = this->m_Dims;
-  newXform->Domain = this->Domain;
-  memcpy( newXform->Spacing, Spacing, sizeof( newXform->Spacing ) );
-  memcpy( newXform->InverseSpacing, InverseSpacing, sizeof( newXform->InverseSpacing ) );
+  newXform->m_Domain = this->m_Domain;
+  newXform->m_Spacing = this->m_Spacing;
+  newXform->m_InverseSpacing = this->m_InverseSpacing;
   newXform->m_Offset = this->m_Offset;
 
   if ( this->m_ActiveFlags ) 
@@ -222,7 +242,7 @@ SplineWarpXform::CloneVirtual() const
     newXform->m_InitialAffineXform = AffineXform::SmartPtr( this->m_InitialAffineXform->Clone() );
     }
   
-  newXform->GlobalScaling = this->GlobalScaling;
+  newXform->m_GlobalScaling = this->m_GlobalScaling;
 
   newXform->nextI = this->nextI;
   newXform->nextJ = this->nextJ;
@@ -235,17 +255,9 @@ SplineWarpXform::CloneVirtual() const
 
   newXform->VolumeDims = this->VolumeDims;
 
-  newXform->gX = this->gX;
-  newXform->gY = this->gY;
-  newXform->gZ = this->gZ;
-
-  newXform->splineX = this->splineX;
-  newXform->splineY = this->splineY;
-  newXform->splineZ = this->splineZ;
-
-  newXform->dsplineX = this->dsplineX;
-  newXform->dsplineY = this->dsplineY;
-  newXform->dsplineZ = this->dsplineZ;
+  newXform->m_GridOffsets = this->m_GridOffsets;
+  newXform->m_GridSpline = this->m_GridSpline;
+  newXform->m_GridDerivSpline = this->m_GridDerivSpline;
 
   return newXform;
 }
@@ -255,7 +267,7 @@ SplineWarpXform::Refine()
 {
   if ( !this->m_ParameterVector ) return;
 
-  Self::IndexType newDims;
+  Self::ControlPointIndexType newDims;
   for ( int dim=0; dim<3; ++dim ) 
     newDims[dim] = 2 * this->m_Dims[dim] - 3;
 
@@ -268,7 +280,7 @@ SplineWarpXform::Refine()
   Types::Coordinate newSpacing[3];
   for ( int dim=0; dim<3; ++dim ) 
     {
-    newSpacing[dim] = Domain[dim] / (newDims[dim]-3);
+    newSpacing[dim] = this->m_Domain[dim] / (newDims[dim]-3);
     }
 
   // no linear refinement here
@@ -346,7 +358,7 @@ SplineWarpXform::Refine()
       }
     }
 
-  NumberOfControlPoints = newNumSamples;
+  this->m_NumberOfControlPoints = newNumSamples;
   this->m_NumberOfParameters = newNumCoefficients;
 
   this->m_ParameterVector = newParameters;
@@ -358,9 +370,9 @@ SplineWarpXform::Refine()
   for ( int dim=0; dim<3; ++dim ) 
     {
     assert( this->m_Dims[dim] > 1 );
-    Spacing[dim] = newSpacing[dim];
-    InverseSpacing[dim] = 1.0 / Spacing[dim];
-    m_Offset[dim] = -Spacing[dim];
+    this->m_Spacing[dim] = newSpacing[dim];
+    this->m_InverseSpacing[dim] = 1.0 / this->m_Spacing[dim];
+    m_Offset[dim] = -this->m_Spacing[dim];
     }
   
   // MUST do this AFTER acutal refinement, as precomputed increments are used
@@ -385,57 +397,61 @@ SplineWarpXform::Refine()
   this->UnRegisterVolume();
 }
 
-void 
-SplineWarpXform::GetVolumeOfInfluence 
-( const size_t idx, const Self::SpaceVectorType& fromVol, const Self::SpaceVectorType& toVol,
-  Self::SpaceVectorType& fromVOI, Self::SpaceVectorType& toVOI, const int fastMode ) const
+UniformVolume::CoordinateRegionType
+SplineWarpXform::GetVolumeOfInfluence( const size_t idx, const UniformVolume::CoordinateRegionType& domain, const bool fastMode ) const
 {
+  Self::SpaceVectorType regionFrom, regionTo;
+  
   int relIdx = idx / 3;
 
   const int xyz[3] = { ( relIdx %  this->m_Dims[0] ), 
-		       ( (relIdx /  this->m_Dims[0]) % this->m_Dims[1] ), 
-		       ( (relIdx /  this->m_Dims[0]) / this->m_Dims[1] ) };
+		       ( (relIdx / this->m_Dims[0]) % this->m_Dims[1] ), 
+		       ( (relIdx / this->m_Dims[0]) / this->m_Dims[1] ) };
   
-  Types::Coordinate xyzLow[3], xyzUp[3];
+  FixedVector<3,Types::Coordinate> xyzLow, xyzUp;
 
-  if ( (fastMode==1) || (this->m_FastMode && (fastMode<0)) ) 
+  if ( fastMode ) 
     {
     for ( int dim = 0; dim < 3; ++dim )
       {
-      xyzLow[dim] = Spacing[dim] * std::max( 0, xyz[dim]-2 );
-      xyzUp[dim] = Spacing[dim] * std::min( this->m_Dims[dim]-3, xyz[dim] );
+      xyzLow[dim] = this->m_Spacing[dim] * std::max( 0, xyz[dim]-2 );
+      xyzUp[dim] = this->m_Spacing[dim] * std::min( this->m_Dims[dim]-3, xyz[dim] );
       }
     } 
   else
     {
     for ( int dim = 0; dim < 3; ++dim )
       {
-      xyzLow[dim] = Spacing[dim] * std::max( 0, xyz[dim]-3 );
-      xyzUp[dim] = Spacing[dim] * std::min( this->m_Dims[dim]-2, xyz[dim]+1 );
+      xyzLow[dim] = this->m_Spacing[dim] * std::max( 0, xyz[dim]-3 );
+      xyzUp[dim] = this->m_Spacing[dim] * std::min( this->m_Dims[dim]-2, xyz[dim]+1 );
       }
     }
   
   for ( int dim = 0; dim < 3; ++dim )
     {
-    fromVOI[dim] = std::min( toVol[dim], std::max( xyzLow[dim], fromVol[dim]) );
-    toVOI[dim] = std::max( fromVol[dim], std::min( xyzUp[dim], toVol[dim]) ); 
+    regionFrom[dim] = std::min( domain.To()[dim], std::max( xyzLow[dim], domain.From()[dim]) );
+    regionTo[dim] = std::max( domain.From()[dim], std::min( xyzUp[dim], domain.To()[dim]) ); 
     }
+
+  return UniformVolume::CoordinateRegionType( regionFrom, regionTo );
 }
 
 void
 SplineWarpXform::RegisterVolumeAxis 
-( const DataGrid::IndexType::ValueType dim, const Types::Coordinate delta, const Types::Coordinate origin, const int cpgDim, const Types::Coordinate invCpgSpacing,
-  std::vector<int>& g, std::vector<Types::Coordinate>& spline, std::vector<Types::Coordinate>& dspline )
+( const DataGrid::IndexType::ValueType dim, const Types::Coordinate delta, const Types::Coordinate origin, const int cpgDim, const size_t ofs, const Types::Coordinate invCpgSpacing,
+  std::vector<int>& gIdx, std::vector<int>& gOfs, std::vector<Types::Coordinate>& spline, std::vector<Types::Coordinate>& dspline )
 {
-  g.resize( dim+1 );
+  gIdx.resize( dim+1 );
+  gOfs.resize( dim+1 );
   spline.resize( 4*dim );
   dspline.resize( 4*dim );
 
   for ( int idx=0; idx<dim; ++idx ) 
     {
     const Types::Coordinate r = invCpgSpacing * (origin + delta * idx);
-    g[idx] = std::min( static_cast<int>( r ), cpgDim-4 );
-    const Types::Coordinate f = r - g[idx];
+    gIdx[idx] = std::min( static_cast<int>( r ), cpgDim-4 );
+    gOfs[idx] = gIdx[idx] * ofs;
+    const Types::Coordinate f = r - gIdx[idx];
     for ( int k = 0; k < 4; ++k ) 
       {
       spline[4*idx+k] = CubicSpline::ApproxSpline( k, f );
@@ -443,46 +459,39 @@ SplineWarpXform::RegisterVolumeAxis
       }
     }
   // guard element
-  g[dim] = -1;
+  gIdx[dim] = gOfs[dim] = -1;
 }
 
 void
 SplineWarpXform::RegisterVolumePoints
 ( const DataGrid::IndexType& volDims, const Self::SpaceVectorType& delta, const Self::SpaceVectorType& origin )
 {
-  this->RegisterVolumeAxis( volDims[0], delta[0], origin[0], this->m_Dims[0], this->InverseSpacing[0], gX, splineX, dsplineX );
-  this->RegisterVolumeAxis( volDims[1], delta[1], origin[1], this->m_Dims[1], this->InverseSpacing[1], gY, splineY, dsplineY );
-  this->RegisterVolumeAxis( volDims[2], delta[2], origin[2], this->m_Dims[2], this->InverseSpacing[2], gZ, splineZ, dsplineZ );
-
-  for ( int idx = 0; idx<volDims[0]; ++idx ) gX[idx] *= nextI;
-  for ( int idx = 0; idx<volDims[1]; ++idx ) gY[idx] *= nextJ;
-  for ( int idx = 0; idx<volDims[2]; ++idx ) gZ[idx] *= nextK;
+  const size_t ijk[3] = { this->nextI, this->nextJ, this->nextK };
+  for ( int axis = 0; axis < 3; ++axis )    
+    this->RegisterVolumeAxis( volDims[axis], delta[axis], origin[axis], this->m_Dims[axis], ijk[axis], this->m_InverseSpacing[axis], 
+			      this->m_GridIndexes[axis], this->m_GridOffsets[axis], this->m_GridSpline[axis], this->m_GridDerivSpline[axis] );
 
   this->VolumeDims = volDims;
 }
 
 void SplineWarpXform::UnRegisterVolume()
 {
-  gX.resize( 0 );
-  gY.resize( 0 );
-  gZ.resize( 0 );
-
-  splineX.resize( 0 );
-  splineY.resize( 0 );
-  splineZ.resize( 0 );
-
-  dsplineX.resize( 0 );
-  dsplineY.resize( 0 );
-  dsplineZ.resize( 0 );
+  for ( int axis = 0; axis < 3; ++axis )
+    {
+    this->m_GridIndexes[axis].resize( 0 );
+    this->m_GridOffsets[axis].resize( 0 );
+    this->m_GridSpline[axis].resize( 0 );
+    this->m_GridDerivSpline[axis].resize( 0 );
+    }
 }
 
-SplineWarpXform::SpaceVectorType& 
-SplineWarpXform::GetDeformedControlPointPosition
-( Self::SpaceVectorType& v, const int x, const int y, const int z) 
-  const 
+SplineWarpXform::SpaceVectorType
+SplineWarpXform::GetDeformedControlPointPosition( const int x, const int y, const int z) const 
 {
+  Self::SpaceVectorType v;
+  
   // Create a pointer to the front-lower-left corner of the c.p.g. cell.
-  const Types::Coordinate* coeff = m_Parameters + 3 * ( (x-1) + this->m_Dims[0] * ((y-1) + this->m_Dims[1] * (z-1)) );  
+  const Types::Coordinate* coeff = m_Parameters + (x-1) * nextI + (y-1) * nextJ + (z-1) * nextK;  
   static const Types::Coordinate spline[3] = { 1.0/6, 4.0/6, 1.0/6 };
 
   for ( int dim = 0; dim<3; ++dim ) 
@@ -519,13 +528,15 @@ SplineWarpXform::GetDeformedControlPointPosition
   return v;
 }
 
-void
+SplineWarpXform::SpaceVectorType
 SplineWarpXform
 ::GetTransformedGrid 
-( Self::SpaceVectorType& v, const int idxX, const int idxY, const int idxZ ) const
+( const int idxX, const int idxY, const int idxZ ) const
 {
-  const Types::Coordinate* coeff = this->m_Parameters + gX[idxX] + gY[idxY] + gZ[idxZ];
-  const Types::Coordinate *spX = &splineX[idxX<<2], *spY = &splineY[idxY<<2], *spZ = &splineZ[idxZ<<2];
+  Self::SpaceVectorType v;
+
+  const Types::Coordinate* coeff = this->m_Parameters + this->m_GridOffsets[0][idxX] + this->m_GridOffsets[1][idxY] + this->m_GridOffsets[2][idxZ];
+  const Types::Coordinate *spX = &this->m_GridSpline[0][idxX<<2], *spY = &this->m_GridSpline[1][idxY<<2], *spZ = &this->m_GridSpline[2][idxZ<<2];
   
   for ( int dim = 0; dim<3; ++dim ) 
     {
@@ -552,6 +563,8 @@ SplineWarpXform
       v[ dim ] = mm;
       ++coeff;
     }
+
+  return v;
 }
 
 void 
@@ -560,8 +573,8 @@ SplineWarpXform::GetTransformedGridRow
   const
 {
   Self::SpaceVectorType *v = vIn;
-  const Types::Coordinate* coeff = m_Parameters + gX[idxX] + gY[idxY] + gZ[idxZ];
-  const Types::Coordinate *spX = &splineX[idxX<<2], *spY = &splineY[idxY<<2], *spZ = &splineZ[idxZ<<2];
+  const Types::Coordinate* coeff = m_Parameters + this->m_GridOffsets[0][idxX] + this->m_GridOffsets[1][idxY] + this->m_GridOffsets[2][idxZ];
+  const Types::Coordinate *spX = &this->m_GridSpline[0][idxX<<2], *spY = &this->m_GridSpline[1][idxY<<2], *spZ = &this->m_GridSpline[2][idxZ<<2];
   
   // precompute the products of B_j(v) and B_k(w) for the 4 x 4 neighborhood
   // in y- and z-direction.
@@ -575,7 +588,7 @@ SplineWarpXform::GetTransformedGridRow
     }
   
   // determine the number of CPG cells on our way along the row
-  const int numberOfCells = (gX[idxX + numPoints - 1] - gX[idxX]) / nextI + 4;
+  const int numberOfCells = (this->m_GridOffsets[0][idxX + numPoints - 1] - this->m_GridOffsets[0][idxX]) / nextI + 4;
   
   // pre-compute the contributions of all control points in y- and z-direction
   // along the way
@@ -630,7 +643,7 @@ SplineWarpXform::GetTransformedGridRow
       ++v;
       // repeat this until we leave current CPG cell.
       } 
-    while ( ( gX[i-1] == gX[i] ) && ( i < lastPoint ) );
+    while ( ( this->m_GridOffsets[0][i-1] == this->m_GridOffsets[0][i] ) && ( i < lastPoint ) );
     
     // we've just left a cell -- shift index of precomputed control points
     // to the next one.
@@ -669,7 +682,7 @@ SplineWarpXform
   
   for ( int dim = 0; dim<3; ++dim ) 
     {
-    r[dim] = InverseSpacing[dim] * v[dim];
+    r[dim] = this->m_InverseSpacing[dim] * v[dim];
     grid[dim] = std::min( static_cast<int>( r[dim] ), this->m_Dims[dim]-4 );
     f[dim] = std::max<Types::Coordinate>( 0, std::min<Types::Coordinate>( 1.0, r[dim] - grid[dim] ) );
     }
@@ -738,18 +751,18 @@ SplineWarpXform
   
   const double energy = 
     // single variable second-order derivatives
-    MathUtil::Square( InverseSpacing[0] ) *
+    MathUtil::Square( this->m_InverseSpacing[0] ) *
     ( J[0][0] * J[0][0] + J[0][1] * J[0][1] + J[0][2] * J[0][2] ) +
-    MathUtil::Square( InverseSpacing[1] ) *
+    MathUtil::Square( this->m_InverseSpacing[1] ) *
     ( J[1][0] * J[1][0] + J[1][1] * J[1][1] + J[1][2] * J[1][2] ) +
-    MathUtil::Square( InverseSpacing[2] ) *
+    MathUtil::Square( this->m_InverseSpacing[2] ) *
     ( J[2][0] * J[2][0] + J[2][1] * J[2][1] + J[2][2] * J[2][2] ) +
     // two-variable mixed derivatives
-    2 * ( InverseSpacing[0] * InverseSpacing[1] *
+    2 * ( this->m_InverseSpacing[0] * this->m_InverseSpacing[1] *
 	  ( K[0][0] * K[0][0] + K[0][1] * K[0][1] + K[0][2] * K[0][2] ) +
-	  InverseSpacing[1] * InverseSpacing[2] *
+	  this->m_InverseSpacing[1] * this->m_InverseSpacing[2] *
 	  ( K[1][0] * K[1][0] + K[1][1] * K[1][1] + K[1][2] * K[1][2] ) +
-	  InverseSpacing[2] * InverseSpacing[0] *
+	  this->m_InverseSpacing[2] * this->m_InverseSpacing[0] *
 	  ( K[2][0] * K[2][0] + K[2][1] * K[2][1] + K[2][2] * K[2][2] )
 	  );
   
@@ -821,18 +834,18 @@ SplineWarpXform
   
   const double energy = 
     // single variable second-order derivatives
-    MathUtil::Square( InverseSpacing[0] ) *
+    MathUtil::Square( this->m_InverseSpacing[0] ) *
     ( J[0][0] * J[0][0] + J[0][1] * J[0][1] + J[0][2] * J[0][2] ) +
-    MathUtil::Square( InverseSpacing[1] ) *
+    MathUtil::Square( this->m_InverseSpacing[1] ) *
     ( J[1][0] * J[1][0] + J[1][1] * J[1][1] + J[1][2] * J[1][2] ) +
-    MathUtil::Square( InverseSpacing[2] ) *
+    MathUtil::Square( this->m_InverseSpacing[2] ) *
     ( J[2][0] * J[2][0] + J[2][1] * J[2][1] + J[2][2] * J[2][2] ) +
     // two-variable mixed derivatives
-    2 * ( InverseSpacing[0] * InverseSpacing[1] *
+    2 * ( this->m_InverseSpacing[0] * this->m_InverseSpacing[1] *
 	  ( K[0][0] * K[0][0] + K[0][1] * K[0][1] + K[0][2] * K[0][2] ) +
-	  InverseSpacing[1] * InverseSpacing[2] *
+	  this->m_InverseSpacing[1] * this->m_InverseSpacing[2] *
 	  ( K[1][0] * K[1][0] + K[1][1] * K[1][1] + K[1][2] * K[1][2] ) +
-	  InverseSpacing[2] * InverseSpacing[0] *
+	  this->m_InverseSpacing[2] * this->m_InverseSpacing[0] *
 	  ( K[2][0] * K[2][0] + K[2][1] * K[2][1] + K[2][2] * K[2][2] )
 	  );
   
@@ -887,8 +900,8 @@ SplineWarpXform::GetGridEnergyDerivative
 
   m_Parameters[param] = oldCoeff;
 
-  upper /= NumberOfControlPoints;
-  lower /= NumberOfControlPoints;
+  upper /= this->m_NumberOfControlPoints;
+  lower /= this->m_NumberOfControlPoints;
 }
 
 Types::Coordinate
@@ -912,9 +925,9 @@ SplineWarpXform::GetInverseConsistencyError
     myVoi = volume->GetWholeImageRegion();
     }
 
-  const int dX = 1 + static_cast<int>( this->Spacing[0] / 2 * volume->m_Delta[AXIS_X] );
-  const int dY = 1 + static_cast<int>( this->Spacing[1] / 2 * volume->m_Delta[AXIS_Y] );
-  const int dZ = 1 + static_cast<int>( this->Spacing[2] / 2 * volume->m_Delta[AXIS_Z] );
+  const int dX = 1 + static_cast<int>( this->m_Spacing[0] / 2 * volume->m_Delta[AXIS_X] );
+  const int dY = 1 + static_cast<int>( this->m_Spacing[1] / 2 * volume->m_Delta[AXIS_Y] );
+  const int dZ = 1 + static_cast<int>( this->m_Spacing[2] / 2 * volume->m_Delta[AXIS_Z] );
 
   const int startX = pVoi->From()[0] - (pVoi->From()[0] % dX);
   const int startY = pVoi->From()[1] - (pVoi->From()[1] % dY);
