@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3221 $
+//  $Revision: 3871 $
 //
-//  $LastChangedDate: 2011-05-13 10:51:19 -0700 (Fri, 13 May 2011) $
+//  $LastChangedDate: 2012-02-15 14:39:46 -0800 (Wed, 15 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -81,15 +81,22 @@ public:
 
   /** Construct new warp from volume size and control grid density.
    */
-  SplineWarpXform( const FixedVector<3,Types::Coordinate>& domain, const Types::Coordinate delta, const AffineXform *initialXform = NULL, const bool exactDelta = false );
+  SplineWarpXform( const Self::SpaceVectorType& domain, const Types::Coordinate delta, const AffineXform *initialXform = NULL, const bool exactDelta = false );
+
+  /** Construct new warp from volume size, grid dimensions and parameters
+   */
+  SplineWarpXform( const Self::SpaceVectorType& domain, const Self::ControlPointIndexType& dims, CoordinateVector::SmartPtr& parameters = CoordinateVector::SmartPtr::Null(), const AffineXform *initialXform = NULL );
 
   /** Initialize warp from volume size and control grid density.
    */
   void Init( const Self::SpaceVectorType& domain, const Types::Coordinate delta, const AffineXform *initialXform = NULL, const bool exactDelta = false );
   
-  /** Construct new warp from volume size, grid dimensions and parameters
+  /** Get region containing all "inside" control point indexes.
+   * The "inside" control points are those for which the transformation can be evaluated.
+   * Since we are using curbic spline basis functions this region is smaller
+   * than that returned by GetAllControlPointsRegion.
    */
-  SplineWarpXform( const FixedVector<3,Types::Coordinate>& domain, const Self::IndexType& dims, CoordinateVector::SmartPtr& parameters, const AffineXform *initialXform = NULL );
+  virtual Self::ControlPointRegionType GetInsideControlPointsRegion() const;
 
   /// Clone and return smart pointer.
   Self::SmartPtr Clone () const 
@@ -97,12 +104,6 @@ public:
     return Self::SmartPtr( this->CloneVirtual() );
   }
 
-  /// Initialize control point positions, potentially with affine displacement.
-  void InitControlPoints( const AffineXform* affineXform = NULL );
-
-  /// Update internal representation.
-  virtual void Update( const bool exactDelta = false );
-
   /** Create inverse transformation.
    * This function returns NULL as there is no explicit inverse of a spline
    * warp transformation.
@@ -246,7 +247,7 @@ public:
       {
       // This is the (real-valued) index of the control point grid cell the
       // given location is in.
-      r[dim] = this->InverseSpacing[dim] * v[dim];
+      r[dim] = this->m_InverseSpacing[dim] * v[dim];
       // This is the actual cell index.
       grid[dim] = std::min<int>( static_cast<int>( r[dim] ), this->m_Dims[dim]-4 );
       // And here's the relative position within the cell.
@@ -291,7 +292,16 @@ public:
   }
 
   /// Get volume influenced by one parameter.
-  virtual void GetVolumeOfInfluence( const size_t idx, const Self::SpaceVectorType&, const Self::SpaceVectorType&, Self::SpaceVectorType&, Self::SpaceVectorType&, const int = -1 ) const;
+  virtual UniformVolume::CoordinateRegionType GetVolumeOfInfluence( const size_t idx /*!< Parameter point index */, 
+								    const UniformVolume::CoordinateRegionType& domain /*!< Underlying image domain.*/ ) const
+  {
+    return this->GetVolumeOfInfluence( idx, domain, this->m_FastMode );
+  }
+  
+  /// Get volume influenced by one parameter with explicit selection of fast vs. accurate mode.
+  virtual UniformVolume::CoordinateRegionType GetVolumeOfInfluence( const size_t idx /*!< Parameter point index */, 
+								    const UniformVolume::CoordinateRegionType& domain /*!< Underlying image domain. */, 
+								    const bool fastMode /*!< Fast mode selector. When 0, "slow", accurate mode is forced. */ ) const;
   
   /// Register the grid points of the deformed uniform or non-uniform volume.
   void RegisterVolume( const UniformVolume* volume )
@@ -299,11 +309,17 @@ public:
     this->RegisterVolumePoints( volume->m_Dims, volume->m_Delta, volume->m_Offset );
   }
 
+  /// Register axes points of the volume to be deformed.
+  void RegisterVolumePoints ( const DataGrid::IndexType&, const Self::SpaceVectorType& );
+
+  /// Register axes points of the volume to be deformed.
+  void RegisterVolumePoints( const DataGrid::IndexType&, const Self::SpaceVectorType&, const Self::SpaceVectorType& );
+
   /// Unegister axes points, ie free all internal data structures.
   void UnRegisterVolume();
   
   /// Get a grid point from the deformed grid.
-  void GetTransformedGrid( Self::SpaceVectorType& v, const int idxX, const int idxY, const int idxZ ) const;
+  Self::SpaceVectorType GetTransformedGrid( const int idxX, const int idxY, const int idxZ ) const;
   
   /// Get a sequence of grid points from the deformed grid. 
   void GetTransformedGridRow( const int numPoints, Self::SpaceVectorType *const v, const int idxX, const int idxY, const int idxZ ) const;
@@ -320,8 +336,11 @@ public:
    * point. It does so more efficiently than applying the transformation to
    * the explicit 3D coordinate of the control point, because most spline
    * coefficients vanish at control points and need not be considered.
+   *\attention The valid range for x,y,z begins at 1, not 0, because we cannot
+   *  compute the transformation for control points outside the domain
+   *  boundary.
    */
-  virtual Self::SpaceVectorType& GetDeformedControlPointPosition( Self::SpaceVectorType&, const int, const int, const int ) const;
+  virtual Self::SpaceVectorType GetDeformedControlPointPosition( const int x, const int y, const int z ) const;
   
   /** Return array of pre deformation vectors.
    * The newly alocated data array contains the control point positions
@@ -344,15 +363,15 @@ public:
   virtual void GetJacobianRow( CoordinateMatrix3x3 *const array, const int x, const int y, const int z, const size_t numberOfPoints ) const;
   
 private:
-  /// Register axes points of the volume to be deformed.
-  void RegisterVolumePoints ( const DataGrid::IndexType&, const Self::SpaceVectorType& );
+  /// Initialize control point positions, potentially with affine displacement.
+  void InitControlPoints( const AffineXform* affineXform = NULL );
 
-  /// Register axes points of the volume to be deformed.
-  void RegisterVolumePoints( const DataGrid::IndexType&, const Self::SpaceVectorType&, const Self::SpaceVectorType& );
+  /// Update internal representation.
+  virtual void Update( const bool exactDelta = false );
 
   /// Register a single axis of the uniform volume to be deformed.
-  void RegisterVolumeAxis ( const DataGrid::IndexType::ValueType, const Types::Coordinate delta, const Types::Coordinate origin, const int, const Types::Coordinate, 
-			    std::vector<int>& g, std::vector<Types::Coordinate>& spline, std::vector<Types::Coordinate>& dspline );
+  void RegisterVolumeAxis ( const DataGrid::IndexType::ValueType, const Types::Coordinate delta, const Types::Coordinate origin, const int, const size_t ofs, const Types::Coordinate, 
+			    std::vector<int>& gIdx, std::vector<int>& gOfs, std::vector<Types::Coordinate>& spline, std::vector<Types::Coordinate>& dspline );
 
   /// Return rigidity constraint based on given Jacobian matrix.
   Types::Coordinate GetRigidityConstraint( const CoordinateMatrix3x3& J ) const;
@@ -364,44 +383,32 @@ protected:
   /// Dimensions of the volume image linked to this transformation.
   DataGrid::IndexType VolumeDims;
 
-  /**\name Precomputed grid indices.
-   * These arrays hold the precomputed grid indices of the deformed grid's
-   * voxels with respect to the control point grid of this deformation.
+  /**\name Precomputed grid index values.
+   * These arrays hold the precomputed grid indexes of the deformed grid's
+   * voxels with respect to the control point grid of this deformation. 
    */
-  //@{
-  /// x-axis.
-  std::vector<int> gX;
-  /// y-axis.
-  std::vector<int> gY;
-  /// z-axis.
-  std::vector<int> gZ;
-  //@}
+  FixedVector< 3,std::vector<int> > m_GridIndexes;
+
+  /**\name Precomputed coefficient array offsets.
+   * These arrays hold the precomputed grid offsets of the deformed grid's
+   * voxels with respect to the control point grid of this deformation. These
+   * values are the grid indexes multiplied by the number of elements to skip in
+   * the coefficient array that corresponds to the respective index.
+   */
+  FixedVector< 3,std::vector<int> > m_GridOffsets;
 
   /**\name Precomputed spline coefficients.
    * These arrays hold the precomputed spline coefficients for deforming the
    * voxel locations in the associated deformed grid.
    */
-  //@{
-  /// x-axis.
-  std::vector<Types::Coordinate> splineX;
-  /// y-axis.
-  std::vector<Types::Coordinate> splineY;
-  /// z-axis.
-  std::vector<Types::Coordinate> splineZ;
-  //@}
-
+  FixedVector< 3,std::vector<Types::Coordinate> > m_GridSpline;
+  
   /**\name Precomputed derivative spline coefficients.
    * These arrays hold the precomputed derivatives of the spline coefficients.
    * This allows for rapid evaluation of the Jacobian determinant.
    */
-  //@{
   /// x-axis.
-  std::vector<Types::Coordinate> dsplineX;
-  /// y-axis.
-  std::vector<Types::Coordinate> dsplineY;
-  /// z-axis.
-  std::vector<Types::Coordinate> dsplineZ;
-  //@}
+  FixedVector< 3,std::vector<Types::Coordinate> > m_GridDerivSpline;
 
   /// Relative offsets of all control points in a 4 x 4 x 4 neighborhood.
   int GridPointOffset[48];
@@ -433,10 +440,13 @@ protected:
   static void GetJacobianConstraintThread( void *const args, const size_t taskIdx, const size_t taskCnt, const size_t, const size_t );
 
   /// Find nearest (after deformation) control point.
-  void FindClosestControlPoint( const Self::SpaceVectorType& v, Self::SpaceVectorType& cp ) const;
+  Self::SpaceVectorType FindClosestControlPoint( const Self::SpaceVectorType& v ) const;
 
   /// Friend declaration.
   friend class SplineWarpXformUniformVolume;
+
+  /// Fitting class is a friend.
+  friend class FitSplineWarpToDeformationField;
 };
 
 } // namespace
