@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2011 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3579 $
+//  $Revision: 3812 $
 //
-//  $LastChangedDate: 2011-11-21 12:58:43 -0800 (Mon, 21 Nov 2011) $
+//  $LastChangedDate: 2012-02-02 15:52:58 -0800 (Thu, 02 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -93,11 +93,10 @@ public:
    * A typed array is built from an existing array.
    *\param data Pointer to the array of values to be stored.
    *\param datasize Number of elements in the data array.
-   *\param freeArray Tag of the memory handler that has to be used for freeing
-   * the array after use.
    *\param paddingflag Flag that indicates whether there are missing elements in
    * the existing data array.
    *\param paddingData Value that marks missing elements in the data array if "paddingFlag" is true.
+   *\param deallocator Pointer to deallocator object, or NULL if deallocation of data block is handled by another object.
    */
   TemplateArray ( void *const data, const size_t datasize, const bool paddingflag, const void* paddingData, const Memory::DeallocatorFunctionPointer deallocator = NULL ) 
   {
