@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3518 $
+//  $Revision: 3812 $
 //
-//  $LastChangedDate: 2011-10-27 13:38:23 -0700 (Thu, 27 Oct 2011) $
+//  $LastChangedDate: 2012-02-02 15:52:58 -0800 (Thu, 02 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -95,6 +95,7 @@ public:
    * array's memory if and only if this flag is true.
    *\param paddingFlag If this flag is not zero, padding data exists in the array.
    *\param paddingData Value used for padding data.
+   *\param deallocator Pointer to a deallocator object that should be used to free the given data block (or NULL if memory is handled by another object)
    *\return A pointer to a new typed array object, or NULL if an error 
    * occurred.
    */
