@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3647 $
+//  $Revision: 3866 $
 //
-//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-02-15 10:58:02 -0800 (Wed, 15 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -47,7 +47,6 @@
 #include <Base/cmtkAffineXform.h>
 #include <Base/cmtkBitVector.h>
 
-#include <Base/cmtkControlPoint.h>
 #include <Base/cmtkMatchedLandmarkList.h>
 
 #include <System/cmtkSmartPtr.h>
@@ -75,24 +74,29 @@ public:
   /// Smart pointer to const WarpXform
   typedef SmartConstPointer<Self> SmartConstPtr;
 
-  /// Grid index type.
-  typedef FixedVector<3,int> IndexType;
-  
+  /// Region type.
+  typedef Region<3,int> ControlPointRegionType;
+
+  /// Index type.
+  typedef ControlPointRegionType::IndexType ControlPointIndexType;
+
   /// Dimensions of control point grid.
-  Self::IndexType m_Dims;
+  Self::ControlPointIndexType m_Dims;
 
   /// Domain of control point grid in world coordinates.
-  FixedVector<3,Types::Coordinate> Domain;
+  Self::SpaceVectorType m_Domain;
 
   /// Array of spacings between the control points.
-  Types::Coordinate Spacing[3];
+  Self::SpaceVectorType m_Spacing;
 
   /// Array of spacings between the control points.
   Self::SpaceVectorType m_Offset;
 
   /// Get global scaling factor.
   virtual Types::Coordinate GetGlobalScaling() const
-  { return this->GlobalScaling; }
+  { 
+    return this->m_GlobalScaling;
+  }
 
   /// Initial affine transformation.
   cmtkGetSetMacro(AffineXform::SmartPtr,InitialAffineXform);
@@ -111,18 +115,24 @@ public:
   /// Get number of control points.
   size_t GetNumberOfControlPoints() const
   {
-    return this->NumberOfControlPoints;
+    return this->m_NumberOfControlPoints;
   }
 
+  /// Get offset of a control point from its index.  
+  size_t GetOffsetFromIndex( const Self::ControlPointIndexType& index ) const
+  {
+    return index[0] * nextI + index[1] * this->nextJ + index[2] * nextK;
+  }
+  
 protected:
   /// Number of control points.
-  size_t NumberOfControlPoints;
+  size_t m_NumberOfControlPoints;
 
   /** Inverted spacings between the control points.
    * These values are used for multiplication instead of division by those in
    * Spacing[].
    */
-  Types::Coordinate InverseSpacing[3];
+  Self::SpaceVectorType m_InverseSpacing;
 
   /// Number of edge planes in the control point grid to keep unmoved.
   cmtkGetSetMacro(unsigned int,IgnoreEdge);
@@ -131,10 +141,10 @@ protected:
   cmtkGetSetMacroDefault(bool,FastMode,true);
 
   /// Precomputed global scaling of initial affine transformation.
-  Types::Coordinate GlobalScaling;
+  Types::Coordinate m_GlobalScaling;
 
   /// Stored scale factors of the initial affine transformation.
-  Types::Coordinate InverseAffineScaling[3];
+  Self::SpaceVectorType m_InverseAffineScaling;
 
   /// Offset of next control grid column.
   int nextI;
@@ -161,28 +171,42 @@ public:
   /// Default constructor.
   WarpXform () : 
     m_InitialAffineXform( NULL ), 
-    NumberOfControlPoints( 0 ),
+    m_NumberOfControlPoints( 0 ),
     m_ActiveFlags( NULL )
   { 
     this->m_IgnoreEdge = 0; 
     this->m_FastMode = false; 
     this->m_Dims[0] = this->m_Dims[1] = this->m_Dims[2] = 0;
-    this->InverseSpacing[0] = this->InverseSpacing[1] = this->InverseSpacing[2] = 0.0;
+    this->m_InverseSpacing[0] = this->m_InverseSpacing[1] = this->m_InverseSpacing[2] = 0.0;
   }
 
   /// Destructor.
   virtual ~WarpXform () {}
 
   /// Initialized internal data structures for new control point grid.
-  virtual void InitGrid( const FixedVector<3,Types::Coordinate>& domain, const Self::IndexType& dims );
+  virtual void InitGrid( const FixedVector<3,Types::Coordinate>& domain, const Self::ControlPointIndexType& dims );
+
+  /// Get region containing all control point indexes.
+  virtual Self::ControlPointRegionType GetAllControlPointsRegion() const;
+
+  /** Get region containing all "inside" control point indexes.
+   * The "inside" control points are those for which the transformation can be evaluated.
+   * For higher-order interpolation kernels, e.g., cubic spline, this region may be smaller
+   * than that returned by GetAllControlPointsRegion, but for general transformations we
+   * default to return the same region here.
+   */
+  virtual Self::ControlPointRegionType GetInsideControlPointsRegion() const
+  {
+    return this->GetAllControlPointsRegion();
+  }
 
   /// Check whether coordinate is in domain of transformation.
   virtual bool InDomain( const Self::SpaceVectorType& v ) const 
   {
     return 
-      ( v[0] >= 0 ) && ( v[0] <= Domain[0] ) &&
-      ( v[1] >= 0 ) && ( v[1] <= Domain[1] ) &&
-      ( v[2] >= 0 ) && ( v[2] <= Domain[2] );
+      ( v[0] >= 0 ) && ( v[0] <= this->m_Domain[0] ) &&
+      ( v[1] >= 0 ) && ( v[1] <= this->m_Domain[1] ) &&
+      ( v[2] >= 0 ) && ( v[2] <= this->m_Domain[2] );
   }
   
   /// Update internal representation.
@@ -214,34 +238,35 @@ public:
     const unsigned int idx, const Types::Coordinate step );
   
   /// Get the original position of a control point.
-  virtual void GetOriginalControlPointPosition( Self::SpaceVectorType& cp, const Types::Coordinate x, const Types::Coordinate y, const Types::Coordinate z) const 
+  virtual Self::SpaceVectorType GetOriginalControlPointPosition( const Types::Coordinate x, const Types::Coordinate y, const Types::Coordinate z) const 
   { 
-    cp[0] = this->m_Offset[0] + x*this->Spacing[0];
-    cp[1] = this->m_Offset[1] + y*this->Spacing[1];
-    cp[2] = this->m_Offset[2] + z*this->Spacing[2];
+    Self::SpaceVectorType cp;
+    cp[0] = this->m_Offset[0] + x*this->m_Spacing[0];
+    cp[1] = this->m_Offset[1] + y*this->m_Spacing[1];
+    cp[2] = this->m_Offset[2] + z*this->m_Spacing[2];
+    return cp;
   }
   
   /// Get the original position of a control point by index.
-  virtual void GetOriginalControlPointPositionByOffset( Self::SpaceVectorType& v, const size_t offset ) const 
+  virtual Self::SpaceVectorType GetOriginalControlPointPositionByOffset( const size_t offset ) const 
   {
-    this->GetOriginalControlPointPosition( v, offset % this->m_Dims[0], (offset % (this->m_Dims[0]*this->m_Dims[1])) / this->m_Dims[0], offset / (this->m_Dims[0]*this->m_Dims[1]) ); 
+    return this->GetOriginalControlPointPosition( offset % this->m_Dims[0], (offset % (this->m_Dims[0]*this->m_Dims[1])) / this->m_Dims[0], offset / (this->m_Dims[0]*this->m_Dims[1]) ); 
   }
 
   /// Get shifted control point position.
-  virtual void GetShiftedControlPointPosition( Self::SpaceVectorType& v, const int x, const int y, const int z ) const 
+  virtual Self::SpaceVectorType GetShiftedControlPointPosition( const int x, const int y, const int z ) const 
   { 
-    this->GetShiftedControlPointPositionByOffset( v, x + this->m_Dims[0] * (y + this->m_Dims[1] * z ) );
+    return this->GetShiftedControlPointPositionByOffset( x + this->m_Dims[0] * (y + this->m_Dims[1] * z ) );
   }
 
   /// Get shifted control point position by offset.
-  virtual void GetShiftedControlPointPositionByOffset( Self::SpaceVectorType& v, const size_t offset ) const 
+  virtual Self::SpaceVectorType GetShiftedControlPointPositionByOffset( const size_t offset ) const 
   { 
-    for ( size_t i = 0; i<3; ++i )
-      v[i] = this->m_Parameters[offset*3+i];
+    return Self::SpaceVectorType( this->m_Parameters + 3 * offset );
   }
 
   /// Set shifted control point position.
-  virtual void SetShiftedControlPointPositionByOffset( const Self::SpaceVectorType& v, const int x, const int y, const int z ) const 
+  virtual void SetShiftedControlPointPosition( const Self::SpaceVectorType& v, const int x, const int y, const int z ) const 
   { 
     this->SetShiftedControlPointPositionByOffset( v, x + this->m_Dims[0] * (y + this->m_Dims[1] * z ) );
   }
@@ -253,6 +278,13 @@ public:
       this->m_Parameters[idx+offset*3] = v[idx];
   }
 
+  /** Get the deformed position of a transformation control point.
+   *\note This function does not necessarily return the shifted control point position,
+   * but rather it applies the current transformation to the given control
+   * point.
+   */
+  virtual Self::SpaceVectorType GetDeformedControlPointPosition( const int, const int, const int ) const = 0;
+
   /// Get parameter step given a transformed volume size.
   virtual Types::Coordinate GetParamStep( const size_t, const Self::SpaceVectorType& volSize, const Types::Coordinate mmStep = 1 ) const;
 
