@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3713 $
+//  $Revision: 3755 $
 //
-//  $LastChangedDate: 2012-01-10 15:33:21 -0800 (Tue, 10 Jan 2012) $
+//  $LastChangedDate: 2012-01-30 15:29:13 -0800 (Mon, 30 Jan 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -62,7 +62,7 @@ ClassStream::PutWarp
   this->WriteBool ( "absolute", true );
   this->WriteIntArray( "dims", warpXform->m_Dims.begin(), 3 );
   
-  this->WriteCoordinateArray( "domain", warpXform->Domain.begin(), 3 );
+  this->WriteCoordinateArray( "domain", warpXform->m_Domain.begin(), 3 );
   this->WriteCoordinateArray( "origin", warpXform->m_Offset.begin(), 3 );
   this->WriteCoordinateArray( "coefficients", nCoeff, warpXform->m_NumberOfParameters, 3 );
   
@@ -176,7 +176,7 @@ ClassStream::Get
       warpXform = NULL; // linear warp no longer supported
       break;
     case 1: 
-      warpXform = new SplineWarpXform( domain, SplineWarpXform::IndexType( dims ), parameters, initialInverse );
+      warpXform = new SplineWarpXform( domain, SplineWarpXform::ControlPointIndexType( dims ), parameters, initialInverse );
       break;
     };
   
