@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3538 $
+//  $Revision: 3603 $
 //
-//  $LastChangedDate: 2011-10-28 16:14:49 -0700 (Fri, 28 Oct 2011) $
+//  $LastChangedDate: 2011-12-01 14:08:18 -0800 (Thu, 01 Dec 2011) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -62,6 +62,18 @@ public:
    */
   DICOM( const char* path );
 
+  /// Get image dimensions (number of pixels per axis).
+  const FixedVector<3,int> GetDims() const;
+
+  /// Get image pixel size.
+  const FixedVector<3,double> GetPixelSize() const;
+
+  /// Get image origin in scanner coordinates.
+  const FixedVector<3,double> GetImageOrigin() const;
+
+  /// Get image orientation of the i and j grid axes.
+  const FixedVector< 2, FixedVector<3,double> > GetImageOrientation() const;
+
   /** Get pixel data array.
    * The pixel data type is determined automatically based on bits allocated and signed vs. unsigned representation.
    * 
@@ -74,6 +86,11 @@ public:
    */
   TypedArray::SmartPtr GetPixelDataArray( const size_t pixelDataLength );
 
+  /// Demosaic (if necessary) the image and return slice normal vector.
+  const FixedVector<3,double> DemosaicAndGetNormal( const FixedVector< 2, FixedVector<3,double> >& imageOrientation /*!< Image orientation vectors (read-only, for default normal computation)*/, 
+						    FixedVector<3,int>& dims /*!< Image dimensions - these may be modified if the image is a mosaic */, 
+						    TypedArray::SmartPtr& pixelDataArray /*!< Pixel data array - a new array will be returned if the image is a mosaic.*/ );
+  
   /// Get const DICOM dataset.
   const DcmDataset& Dataset() const
   {
