@@ -22,11 +22,11 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3713 $
+//  $Revision: 3890 $
 //
-//  $LastChangedDate: 2012-01-10 15:33:21 -0800 (Tue, 10 Jan 2012) $
+//  $LastChangedDate: 2012-02-17 23:07:13 -0800 (Fri, 17 Feb 2012) $
 //
-//  $LastChangedBy: torstenrohlfing $
+//  $LastChangedBy: torsten_at_home $
 //
 */
 
@@ -298,7 +298,7 @@ TypedStream
     case Self::MODE_APPEND:
       if ( GzFile ) 
 	{
-	if ( 0 == gztell( File) )
+	if ( 0 == gztell( this->GzFile) )
 	  gzprintf( GzFile, "%s\n", GetTypedStreamIdent() );
 	} 
       else
@@ -797,7 +797,7 @@ TypedStream
     {
     for ( int level = 0; level < currentLevel; level++ )
       gzputs( GzFile, "\t" );
-    gzprintf( File, "%s %.*f\n", key, PrecisionDouble, value );
+    gzprintf( this->GzFile, "%s %.*f\n", key, PrecisionDouble, value );
     } 
   else
     {
@@ -944,7 +944,7 @@ TypedStream
     for ( int level = 0; level < currentLevel; level++)
       gzputs( GzFile, "\t" );
     
-    gzprintf( File, "%s ", key );
+    gzprintf( this->GzFile, "%s ", key );
     
     for ( int i = 0; i < size; i++) 
       {
