@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2009 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3647 $
+//  $Revision: 3874 $
 //
-//  $LastChangedDate: 2011-12-22 10:49:31 -0800 (Thu, 22 Dec 2011) $
+//  $LastChangedDate: 2012-02-15 15:19:46 -0800 (Wed, 15 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -36,6 +36,7 @@
 #include <System/cmtkDebugOutput.h>
 #include <System/cmtkFileUtils.h>
 #include <System/cmtkMountPoints.h>
+#include <System/cmtkExitException.h>
 
 #include <IO/cmtkFileFormat.h>
 #include <IO/cmtkClassStream.h>
@@ -91,19 +92,16 @@ XformIO::Read( const char* path )
       return Xform::SmartPtr( warpXform );
     
     stream.Open( realPath, ClassStream::MODE_READ );
-    try
-      {
-      AffineXform affineXform;
-      stream >> affineXform;
-      return Xform::SmartPtr( new AffineXform( affineXform ) );
-      }
-    catch (...)
-      {
-      return Xform::SmartPtr( NULL );
-      }
+
+    AffineXform affineXform;
+    stream >> affineXform;
+    return Xform::SmartPtr( new AffineXform( affineXform ) );
     }
     default:
-      StdErr << "The file/directory " << realPath << " does not seem to be in a supported transformation format\n";
+    {
+    StdErr << "The file/directory " << realPath << " does not seem to be in a supported transformation format\n";
+    throw ExitException( 1 );
+    }
     }
   return Xform::SmartPtr( NULL );
 }
