@@ -20,9 +20,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3710 $
+//  $Revision: 3947 $
 //
-//  $LastChangedDate: 2012-01-10 13:39:39 -0800 (Tue, 10 Jan 2012) $
+//  $LastChangedDate: 2012-02-29 12:40:17 -0800 (Wed, 29 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -130,8 +130,10 @@ cmtk::EchoPlanarUnwarpFunctional::GetJacobianMap( const int direction ) const
 #ifndef _OPENMP
   const DataGrid::RegionType region = wholeImageRegion;
 #else // _OPENMP
+  const int sliceFrom = wholeImageRegion.From()[this->m_ReadoutDirection];
+  const int sliceTo = wholeImageRegion.To()[this->m_ReadoutDirection];
 #pragma omp parallel for
-  for ( int slice = wholeImageRegion.From()[this->m_ReadoutDirection]; slice < wholeImageRegion.To()[this->m_ReadoutDirection]; ++slice )
+  for ( int slice = sliceFrom; slice < sliceTo; ++slice )
     {
     DataGrid::RegionType region = wholeImageRegion;
     region.From()[this->m_ReadoutDirection] = slice;
@@ -178,8 +180,10 @@ cmtk::EchoPlanarUnwarpFunctional::MakeGradientImage( const ap::real_1d_array& u,
 #ifndef _OPENMP
   const DataGrid::RegionType region = wholeImageRegion;
 #else // _OPENMP
+  const int sliceFrom = wholeImageRegion.From()[this->m_ReadoutDirection];
+  const int sliceTo = wholeImageRegion.To()[this->m_ReadoutDirection];
 #pragma omp parallel for
-  for ( int slice = wholeImageRegion.From()[this->m_ReadoutDirection]; slice < wholeImageRegion.To()[this->m_ReadoutDirection]; ++slice )
+  for ( int slice = sliceFrom; slice < sliceTo; ++slice )
     {
     DataGrid::RegionType region = wholeImageRegion;
     region.From()[this->m_ReadoutDirection] = slice;
@@ -247,8 +251,10 @@ cmtk::EchoPlanarUnwarpFunctional::ComputeDeformedImage( const ap::real_1d_array&
 #ifndef _OPENMP
   const DataGrid::RegionType region = wholeImageRegion;
 #else // _OPENMP
+  const int sliceFrom = wholeImageRegion.From()[this->m_ReadoutDirection];
+  const int sliceTo = wholeImageRegion.To()[this->m_ReadoutDirection];
 #pragma omp parallel for
-  for ( int slice = wholeImageRegion.From()[this->m_ReadoutDirection]; slice < wholeImageRegion.To()[this->m_ReadoutDirection]; ++slice )
+  for ( int slice = sliceFrom; slice < sliceTo; ++slice )
     {
     DataGrid::RegionType region = wholeImageRegion;
     region.From()[this->m_ReadoutDirection] = slice;
@@ -331,7 +337,7 @@ cmtk::EchoPlanarUnwarpFunctional
   const int phaseEncodeDirection = function.m_PhaseEncodeDirection;
 
   // reset gradient vector
-  for ( size_t i = g.getlowbound(); i <= g.gethighbound(); ++i )
+  for ( int i = g.getlowbound(); i <= g.gethighbound(); ++i )
     g(i) = 0.0;
 
   const UniformVolume& sourceImage = *(function.m_ImageGrid);
@@ -366,8 +372,10 @@ cmtk::EchoPlanarUnwarpFunctional
   const DataGrid::RegionType region = insideRegion;
   {
 #else // _OPENMP
+  const int sliceFrom = wholeImageRegion.From()[function.m_ReadoutDirection];
+  const int sliceTo = wholeImageRegion.To()[function.m_ReadoutDirection];
 #pragma omp parallel for reduction(+:msd)
-  for ( int slice = insideRegion.From()[function.m_ReadoutDirection]; slice < insideRegion.To()[function.m_ReadoutDirection]; ++slice )
+  for ( int slice = sliceFrom; slice < sliceTo; ++slice )
     {
     DataGrid::RegionType region = insideRegion;
     region.From()[function.m_ReadoutDirection] = slice;
@@ -413,8 +421,10 @@ cmtk::EchoPlanarUnwarpFunctional
       const DataGrid::RegionType region = insideRegion;
       {
 #else // _OPENMP
+      const int sliceFrom = insideRegion.From()[function.m_ReadoutDirection];
+      const int sliceTo = insideRegion.To()[function.m_ReadoutDirection];
 #pragma omp parallel for reduction(+:smooth)
-      for ( int slice = insideRegion.From()[function.m_ReadoutDirection]; slice < insideRegion.To()[function.m_ReadoutDirection]; ++slice )
+      for ( int slice = sliceFrom; slice < sliceTo; ++slice )
 	{
 	DataGrid::RegionType region = insideRegion;
 	region.From()[function.m_ReadoutDirection] = slice;
@@ -450,8 +460,10 @@ cmtk::EchoPlanarUnwarpFunctional
 #ifndef _OPENMP
     const DataGrid::RegionType region = insideRegion;
 #else // _OPENMP
+    const int sliceFrom = insideRegion.From()[function.m_ReadoutDirection];
+    const int sliceTo = insideRegion.To()[function.m_ReadoutDirection];
 #pragma omp parallel for reduction(+:fold)
-    for ( int slice = insideRegion.From()[function.m_ReadoutDirection]; slice < insideRegion.To()[function.m_ReadoutDirection]; ++slice )
+    for ( int slice = sliceFrom; slice < sliceTo; ++slice )
       {
       DataGrid::RegionType region = insideRegion;
       region.From()[function.m_ReadoutDirection] = slice;
