@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3281 $
+//  $Revision: 3608 $
 //
-//  $LastChangedDate: 2011-07-26 14:38:18 -0700 (Tue, 26 Jul 2011) $
+//  $LastChangedDate: 2011-12-02 10:47:18 -0800 (Fri, 02 Dec 2011) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -44,10 +44,6 @@
 
 #include <vector>
 
-#ifdef CMTK_USE_MPI
-#  include <mpi.h>
-#endif
-
 namespace
 cmtk
 {
@@ -455,9 +451,7 @@ protected:
   virtual void UpdateProbabilisticSamples();
 
   /** Interpolate all moving images.
-   * By default, this only calls InterpolateImage() for each image. However,
-   * this functionality can be overriden for better efficiency, for example
-   * in the distributed MPI implementation.
+   * By default, this only calls InterpolateImage() for each image. 
    */
   virtual void InterpolateAllImages();
 
@@ -497,14 +491,6 @@ protected:
   /// Update probabilistic sample table..
   virtual bool Wiggle();
 
-#ifdef CMTK_USE_MPI
-  /// MPI process rank.
-  int m_RankMPI;
-
-  /// MPI process count.
-  int m_SizeMPI;
-#endif
-
   /// Prepare data for one image.
   virtual UniformVolume::SmartPtr PrepareSingleImage( UniformVolume::SmartPtr& image );
 
