@@ -2,7 +2,7 @@
 //
 //  Copyright 1997-2010 Torsten Rohlfing
 //
-//  Copyright 2004-2011 SRI International
+//  Copyright 2004-2012 SRI International
 //
 //  This file is part of the Computational Morphometry Toolkit.
 //
@@ -22,9 +22,9 @@
 //  with the Computational Morphometry Toolkit.  If not, see
 //  <http://www.gnu.org/licenses/>.
 //
-//  $Revision: 3517 $
+//  $Revision: 3843 $
 //
-//  $LastChangedDate: 2011-10-27 12:42:15 -0700 (Thu, 27 Oct 2011) $
+//  $LastChangedDate: 2012-02-10 14:49:31 -0800 (Fri, 10 Feb 2012) $
 //
 //  $LastChangedBy: torstenrohlfing $
 //
@@ -171,11 +171,8 @@ protected:
    */
   DataGrid::RegionType *VolumeOfInfluence;
 
-  /// Coordinate of the beginning of the reference colume crop area.
-  Vector3D ReferenceFrom;
-
-  /// Coordinate of the end of the reference colume crop area.
-  Vector3D ReferenceTo;
+  /// Reference volume coordinate domain.
+  UniformVolume::CoordinateRegionType m_ReferenceDomain;
 
   /// Storage for simultaneously retrieving multiple deformed vectors.
   Vector3D *VectorCache;
@@ -401,7 +398,7 @@ public:
     Types::Coordinate fltFrac[3];
 
     int endLineIncrement = ( voi.From()[0] + (DimsX - voi.To()[0]) );
-    int endPlaneIncrement = DimsX * ( voi.From()[1] + (DimsY - voi.To()[2]) );
+    int endPlaneIncrement = DimsX * ( voi.From()[1] + (DimsY - voi.To()[1]) );
     
     const typename VM::Exchange unsetY = this->Metric->DataY.padding();
     *localMetric = *this->Metric;
